@@ -636,6 +636,11 @@ libcalc_call_me_last(void)
 		}
 	}
 
+	/*
+	 * free all globals
+	 */
+	freeglobals();
+
 	/*
 	 * all done
 	 */
