@@ -37,26 +37,15 @@
 NUMBER _qzero_ =	{ { _zeroval_, 1, 0 }, { _oneval_, 1, 0 }, 1, NULL };
 NUMBER _qone_ =		{ { _oneval_, 1, 0  }, { _oneval_, 1, 0 }, 1, NULL };
 NUMBER _qtwo_ =		{ { _twoval_, 1, 0 }, { _oneval_, 1, 0 }, 1, NULL };
-NUMBER _qthree_ =	{ { _threeval_, 1, 0 }, { _oneval_, 1, 0 }, 1, NULL };
-NUMBER _qfour_ =	{ { _fourval_, 1, 0 }, { _oneval_, 1, 0 }, 1, NULL };
 NUMBER _qten_ =		{ { _tenval_, 1, 0 }, { _oneval_, 1, 0 }, 1, NULL };
 NUMBER _qnegone_ =	{ { _oneval_, 1, 1 }, { _oneval_, 1, 0 }, 1, NULL };
 NUMBER _qonehalf_ =	{ { _oneval_, 1, 0 }, { _twoval_, 1, 0 }, 1, NULL };
 NUMBER _qneghalf_ = 	{ { _oneval_, 1, 1 }, { _twoval_, 1, 0 }, 1, NULL };
 NUMBER _qonesqbase_ =	{ { _oneval_, 1, 0 }, { _sqbaseval_, 2, 0 }, 1, NULL };
-NUMBER _qtendivnine_ =	{ { _tenval_, 1, 0 }, { _nineval_, 1, 0 }, 1, NULL };
-NUMBER _qninedivten_ =	{ { _nineval_, 1, 0 }, { _tenval_, 1, 0 }, 1, NULL };
-NUMBER _qthreesixty =	{ { _threesixtyval_, 1, 0 },
-			  { _oneval_, 1, 0 }, 1, NULL };
-NUMBER _qfourhundred =	{ { _fourhundredval_, 1, 0 },
-			  { _oneval_, 1, 0 }, 1, NULL };
-NUMBER _qtwentyfour =	{ { _twentyfourval_, 1, 0 },
-			  { _oneval_, 1, 0 }, 1, NULL };
 
-NUMBER * initnumbs[] = {&_qzero_, &_qone_, &_qtwo_, &_qthree_,
-	&_qfour_, &_qten_, &_qnegone_, &_qonehalf_, &_qneghalf_,
-	&_qonesqbase_, &_qtendivnine_, &_qninedivten_,
-	&_qthreesixty, &_qfourhundred, &_qtwentyfour,
+NUMBER * initnumbs[] = {&_qzero_, &_qone_, &_qtwo_,
+	&_qten_, &_qnegone_, &_qonehalf_, &_qneghalf_,
+	&_qonesqbase_,
 	NULL	/* must be last */
 };
 
