@@ -110,7 +110,8 @@ addglobal(char *name, BOOL isstatic)
 		return NULL;
 	hp = &globalhash[HASHSYM(name, len)];
 	for (sp = *hp; sp; sp = sp->g_next) {
-		if ((sp->g_len == len) && (strcmp(sp->g_name, name) == 0)
+		if ((sp->g_len == len) &&
+		    (strncmp(sp->g_name, name, len+1) == 0)
 			&& (sp->g_filescope == newfilescope)
 			&& (sp->g_funcscope == newfuncscope))
 				return sp;
@@ -145,8 +146,9 @@ findglobal(char *name)
 
 	bestsp = NULL;
 	len = strlen(name);
-	for (sp = globalhash[HASHSYM(name, len)]; sp; sp = sp->g_next) {
-		if ((sp->g_len == len) && !strcmp(sp->g_name, name)) {
+	for (sp = globalhash[HASHSYM(name, len)]; sp != NULL; sp = sp->g_next) {
+		if ((sp->g_len == len) &&
+		   (strncmp(sp->g_name, name, len+1) == 0)) {
 			if ((bestsp == NULL) ||
 				(sp->g_filescope > bestsp->g_filescope) ||
 				 (sp->g_funcscope > bestsp->g_funcscope))
@@ -232,6 +234,7 @@ showallglobals(void)
 		printf("No global variables\n");
 }
 
+
 S_FUNC void
 printtype(VALUE *vp)
 {
@@ -360,6 +363,7 @@ writeglobals(char *name)
 	return 0;
 }
 
+
 /*
  * Free all non-null global and visible static variables
  */
