@@ -493,6 +493,7 @@ zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res)
 	HALF curhalf;		/* current word of power */
 	unsigned int curpow;	/* current low power */
 	unsigned int curbit;	/* current bit of low power */
+	BOOL free_z1;		/* TRUE => need to free z1 */
 	int i;
 
 	if (zisneg(z3) || ziszero(z3)) {
@@ -535,17 +536,22 @@ zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res)
 	 * within the modulo range.  Then check for zero or one specially.
 	 */
 	ztmp.len = 0;
+	free_z1 = FALSE;
 	if (zisneg(z1) || zrel(z1, z3) >= 0) {
 		zmod(z1, z3, &ztmp, 0);
+		zfree(z1);
 		z1 = ztmp;
+		free_z1 = TRUE;
 	}
 	if (ziszero(z1)) {
+		zfree(z1);
 		if (ztmp.len)
 			zfree(ztmp);
 		*res = _zero_;
 		return;
 	}
 	if (zisone(z1)) {
+		zfree(z1);
 		if (ztmp.len)
 			zfree(ztmp);
 		*res = _one_;
@@ -598,10 +604,12 @@ zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res)
 			 * future use.
 			 */
 			if (pp->v == NULL) {
-				if (curpow & 0x1)
+				if (curpow & 0x1) {
 					zcopy(z1, &modpow);
-				else
+					free_z1 = FALSE;
+				} else {
 					modpow = _one_;
+				}
 
 				for (curbit = 0x2;
 				     curbit <= curpow;
@@ -667,8 +675,7 @@ zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res)
 		}
 
 		for (pp = &lowpowers[2]; pp <= &lowpowers[POWNUMS-1]; pp++) {
-			if (pp->v != NULL)
-				freeh(pp->v);
+			zfree(*pp);
 		}
 		*res = ans;
 		if (ztmp.len)
@@ -689,6 +696,9 @@ zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res)
 			powermodredc = zredcalloc(z3);
 		rp = powermodredc;
 		zredcencode(rp, z1, &temp);
+		if (free_z1 == TRUE) {
+			zfree(z1);
+		}
 		zredcpower(rp, temp, z2, &z1);
 		zfree(temp);
 		zredcdecode(rp, z1, res);
@@ -727,10 +737,12 @@ zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res)
 		 * calculate it and remember it in the table for future use.
 		 */
 		if (pp->v == NULL) {
-			if (curpow & 0x1)
+			if (curpow & 0x1) {
 				zcopy(z1, &modpow);
-			else
+				free_z1 = FALSE;
+			} else {
 				modpow = _one_;
+			}
 
 			for (curbit = 0x2; curbit <= curpow; curbit *= 2) {
 				pp = &lowpowers[curbit];
@@ -789,12 +801,14 @@ zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res)
 	}
 
 	for (pp = &lowpowers[2]; pp <= &lowpowers[POWNUMS-1]; pp++) {
-		if (pp->v != NULL)
-			freeh(pp->v);
+		zfree(*pp);
 	}
 	*res = ans;
 	if (ztmp.len)
 		zfree(ztmp);
+	if (free_z1 == TRUE) {
+		zfree(z1);
+	}
 }
 
 /*
@@ -1836,8 +1850,7 @@ zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res)
 	}
 
 	for (pp = lowpowers; pp < &lowpowers[POWNUMS]; pp++) {
-		if (pp->len)
-			freeh(pp->v);
+		zfree(*pp);
 	}
 	if (sign && !ziszero(ans)) {
 		zsub(rp->mod, ans, res);
